Weather.MenuCities = newClass(Wdgt.Menu, {
	constructor: function(menu, behaviour) {
		this.owner = behaviour.owner;
		behaviour.name = 'cities';
		behaviour.max_height = 100;
		this.data = behaviour.data;
		this.input = behaviour.input;
		this.disable_suggestor = false;
		this.mode = '';
		this.opera = Wdgt.platform.name == 'opera';
		this.linux = Wdgt.platform.os == 'linux';
		
		this.current = 0;
		this.range = new y5.Range(this.input);
		var result = this.constructor.prototype.constructor.call(this, menu, behaviour);
		
//		this.requestTimer = 0;
//		this.abortTimer = 0;
//		this.ajax = null;
		
//		this.underline = new Search.Underline(this.input);
		
		var context = this;

		// события
		
		y5.Events.observe('keydown', this.onInputKeyDown, this.input, true, this);
		y5.Events.observe('keyup', this.onInputKeyUp, this.input, true, this);
		//y5.Events.observe('blur', this.onDone, this.input, true, this);

		y5.ShortCut.down([{key: y5.ShortCut.ENTER}], this.onEnter, this.input, {checkTarget: false, preventDefault: true, context: this});
		var ctrl = (Wdgt.platform.name == 'opera') && (Wdgt.platform.os != 'mac');
		
		// сообщения
//		new y5.Observer('item-hover', this.onItemHover, this, true, this);
		new y5.Observer('item-click', this.onItemClick, this, true, this);
		new y5.Observer('item-key', this.onItemKey, this, true, this);
		return result;
	},
	onEnter: function(e) {
		this.onDone(e);
		y5.Notify('city-menu-done', this.owner);
	},
	onDone: function(e) {
		var value = y5.Strings.normalizeSpace(this.input.value);
		
		var cities = this.data.cities;//this.owner.data.cities;
		if(cities) {
			this.current = 0;
			for(var id in cities)
				if(cities[id].title.toLowerCase() == value.toLowerCase()) {
					this.current = id;
					break;
				}
			if(this.current == 0)
				this.current = this.data.geo_target_weather;
				this.input.value = cities[this.current].title;
		}
		//if(this.behaviour.avto_save)
		//	this.store();
		
		this.hideAllMenu();
		try {
			this.range.select(0, this.input.value.length);
			this.input.focus();
		} catch(e) {}
	},
	store: function() {
		this.onDone();
		//this.owner.data.setWeatherId(this.current);
		y5.Notify('update', this.data, {id: this.current});
	},
	visibility: function(show) {
		if(!show && this.isVisible())
			this.mode = '';
		var result = this.constructor.prototype.visibility.call(this, show);
		return result;
	},
	filtered: function(part) {
		var cities = this.data.cities;
		part = y5.Strings.normalizeSpace(part.toLowerCase());
		var filtered = [];
		if(part != '') {
			for(var id in cities) {
				if(cities[id].title.toLowerCase().indexOf(part) == 0)
					filtered.push(cities[id]);
			}
			
			function rule(a, b) {return a.title > b.title ? 1 : -1;}
			filtered.sort(rule);
		}
		return filtered;
	},
	
	isArrowKey: function(e) {
		switch(e.keyCode) {
			case y5.ShortCut.UP_ARROW:
			case y5.ShortCut.DOWN_ARROW:
			case y5.ShortCut.LEFT_ARROW:
			case y5.ShortCut.RIGHT_ARROW:
				return true;
			break;
		}
		return false;
	},
	isDelKey: function(e) {
		return (e.keyCode == y5.ShortCut.BACKSPACE) || (e.keyCode == y5.ShortCut.DELETE);
	},
	isEnterKey: function(e) {
		return (e.keyCode == y5.ShortCut.ENTER);
	},
	isModifierKey: function(e) {
		switch(e.keyCode) {
			case y5.ShortCut.SHIFT:
			case y5.ShortCut.CTRL:
			case y5.ShortCut.ALT:
			case y5.ShortCut.CAPS_LOCK:
				return true;
			break;
		}
		return false;		
	},
	operaInputFilter: function() {
		var value = this.input.value;
		if(value != value.replace(/\x1e/gm, '')) {
			this.input.value = value.replace(/\x1e/gm, '');
		}	
	},
	/* события */
	onSelect: function() {
		this.hideAllMenu();
		
		
		
		this.range.select(0, this.input.value.length);
		this.input.focus();
	},
	onInputKeyDown: function(e) {
		if (this.opera) {
			var context = this;
			function filter() {context.operaInputFilter.call(context);}
			setTimeout(filter, 1);
			
			if (this.linux)
				setTimeout(function() {context.onInputKeyUp.call(context, e);}, 1);
		}
	},
	onInputKeyUp: function(e) {
		if(this.opera)
			this.operaInputFilter();
	
		if(this.isArrowKey(e) || this.isEnterKey(e) || this.isModifierKey(e))
			return;
				
		this.disable_suggestor = this.isDelKey(e);
		
		this.type_length = this.input.value.length;

		var value = this.input.value;
		if(y5.Strings.normalizeSpace(value) == '') {
			clearTimeout(this.suggestTimer);
			this.hide();
			return;
		}
		
		this.suggestFilter();
		
		this.mode = 'suggestor';
	},
	suggestFilter: function() {
		clearTimeout(this.suggestTimer);
		var context = this;
		function onTimeout() {context.suggest.call(context);}
		this.suggestTimer = setTimeout(onTimeout, 600);
	},
	suggest: function() {
		var value = this.input.value;
		var items = [];
		var filtered = this.filtered(value);
		var n = filtered.length;
		for(var i = 0; i < n; i++) {
			items.push({
				title: filtered[i].title,
				className: 'city',
				context: this
			});
		}
		
		if(items.length > 0) {
			this.create(items);
			this.show();
			if(!this.disable_suggestor) {
				this.selectFirstItem();
				this.onItemKey();
			}
		} else {
			this.hide();
			this.menu = [];
		}
	},
	/* cообщения */
	onItemClick: function() {
		var active = this.getActive();
		if(typeof active.menu.menu != 'undefined')
			return;
		this.input.value = active.menu.title;
		this.onDone();
	},
	onItemKey: function() {
		var active = this.getActive();
		if(!(active && active.menu))
			return;
		if(typeof active.menu.menu != 'undefined')
			return;
					
		this.input.value = active.menu.title;
		this.range.select(this.type_length, this.input.value.length);
		
		if(Wdgt.platform.name == 'opera') {
			var context = this;
			function select() {context.range.select(context.type_length, context.input.value.length);}
			setTimeout(select, 1);
		}
	}
});